// Rattail CEView.cpp : implementation of the CRattailCEView class
//

#include "stdafx.h"
#include "Rattail CE.h"

#include "Rattail CEDoc.h"
#include "Rattail CEView.h"

#include "ScanCApi.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

enum tagUSERMSGS
{
	UM_SCAN	= WM_USER + 0x200,
	UM_STARTSCANNING,
	UM_STOPSCANNING
};

/////////////////////////////////////////////////////////////////////////////
// CRattailCEView

IMPLEMENT_DYNCREATE(CRattailCEView, CFormView)

BEGIN_MESSAGE_MAP(CRattailCEView, CFormView)
	//{{AFX_MSG_MAP(CRattailCEView)
	ON_NOTIFY(NM_CLICK, IDC_SCANS, OnClickScans)
	ON_EN_SETFOCUS(IDC_SCANCODE, OnScancodeSetfocus)
	ON_EN_KILLFOCUS(IDC_SCANCODE, OnScancodeKillfocus)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRattailCEView construction/destruction

CRattailCEView::CRattailCEView()
	: CFormView(CRattailCEView::IDD)
{
	//{{AFX_DATA_INIT(CRattailCEView)
	//}}AFX_DATA_INIT

	m_bInited = false;
	m_lpScanBuffer = NULL;
	OpenScanner();
}

CRattailCEView::~CRattailCEView()
{
	CloseScanner();
}

void CRattailCEView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRattailCEView)
	DDX_Control(pDX, IDC_STOP_UNIT, m_StopUnit);
	DDX_Control(pDX, IDC_UNIT_QUANTITY, m_UnitQuantity);
	DDX_Control(pDX, IDC_STOP_CASE, m_StopCase);
	DDX_Control(pDX, IDC_CASE_QUANTITY, m_CaseQuantity);
	DDX_Control(pDX, IDC_SCANS, m_Scans);
	DDX_Control(pDX, IDC_SCANCODE, m_Scancode);
	//}}AFX_DATA_MAP
}

BOOL CRattailCEView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs
	return CFormView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CRattailCEView diagnostics

#ifdef _DEBUG
void CRattailCEView::AssertValid() const
{
	CFormView::AssertValid();
}

void CRattailCEView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CRattailCEDoc* CRattailCEView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CRattailCEDoc)));
	return (CRattailCEDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CRattailCEView message handlers

void CRattailCEView::OnClickScans(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// TODO: Add your control notification handler code here
	
	*pResult = 0;
}

void CRattailCEView::OnInitialUpdate() 
{
	CFormView::OnInitialUpdate();
	
	// TODO: Add your specialized code here and/or call the base class
	if (! m_bInited) {
		m_Scans.InsertColumn(0, L"Scan Code");
		m_Scans.SetColumnWidth(0, 160);
		m_Scans.InsertColumn(1, L"Case", LVCFMT_RIGHT);
		m_Scans.SetColumnWidth(1, 50);
		m_Scans.InsertColumn(2, L"Unit", LVCFMT_RIGHT);
		m_Scans.SetColumnWidth(2, 50);
		m_CaseQuantity.SetWindowText(L"0");
		m_StopUnit.SetCheck(BST_CHECKED);
		m_UnitQuantity.SetWindowText(L"1");
		m_bInited = true;
	}
	m_Scancode.SetFocus();
}

bool CRattailCEView::ProcessScan()
{
	if (m_StopCase.GetCheck() == BST_CHECKED) {
		m_CaseQuantity.SetSel(0, -1);
		m_CaseQuantity.SetFocus();
		return false;
	}

	if (m_StopUnit.GetCheck() == BST_CHECKED) {
		m_UnitQuantity.SetSel(0, -1);
		m_UnitQuantity.SetFocus();
		return false;
	}

	return StoreScanData();
}

BOOL CRattailCEView::PreTranslateMessage(MSG* pMsg) 
{
	LPSCAN_BUFFER lpScanBuffer;
	CString scandata;

	if (pMsg->message == UM_SCAN) {
		m_dwRequestID = NULL; // clear this so logic elsewhere isn't confused
		
		lpScanBuffer = (LPSCAN_BUFFER) pMsg->lParam;
		if (SCNBUF_GETSTAT(lpScanBuffer) == E_SCN_SUCCESS) {
			scandata = (LPCTSTR) SCNBUF_GETDATA(lpScanBuffer);
			m_Scancode.SetWindowText(scandata.Left(scandata.GetLength() - 1));
			if (ProcessScan()) {
				EnableScanner();
			}
		}

	} else if (pMsg->message == WM_KEYUP) {
		if (pMsg->wParam == VK_RETURN) {

			if (pMsg->hwnd == m_Scancode.m_hWnd) {
				ProcessScan();

			} else if (pMsg->hwnd == m_CaseQuantity.m_hWnd) {
				if (m_StopUnit.GetCheck() == BST_CHECKED) {
					m_UnitQuantity.SetSel(0, -1);
					m_UnitQuantity.SetFocus();
				} else {
					StoreScanData();
				}

			} else if (pMsg->hwnd == m_UnitQuantity.m_hWnd) {
				StoreScanData();
			}
		}
	}
	return CFormView::PreTranslateMessage(pMsg);
}

bool CRattailCEView::StoreScanData()
{
	CString scancode, cases, units;
	int row;
	CRattailCEDoc* doc;

	m_Scancode.GetWindowText(scancode);
	scancode.TrimLeft();
	scancode.TrimRight();
	if (scancode.IsEmpty()) {
		m_Scancode.SetSel(0, -1);
		m_Scancode.SetFocus();
		return false;
	}

	m_CaseQuantity.GetWindowText(cases);
	cases.TrimLeft();
	cases.TrimRight();
	if (cases == "0") {
		cases.Empty();
	}

	m_UnitQuantity.GetWindowText(units);
	units.TrimLeft();
	units.TrimRight();
	if (units == "0") {
		units.Empty();
	}

	if (cases.IsEmpty() && units.IsEmpty()) {
		if (m_StopCase.GetCheck() == BST_CHECKED) {
			m_CaseQuantity.SetSel(0, -1);
			m_CaseQuantity.SetFocus();
		} else if (m_StopUnit.GetCheck() == BST_CHECKED) {
			m_UnitQuantity.SetSel(0, -1);
			m_UnitQuantity.SetFocus();
		} else {
			m_CaseQuantity.SetSel(0, -1);
			m_CaseQuantity.SetFocus();
		}
		return false;
	}

	doc = GetDocument();
	doc->AddScanDatum(scancode, cases, units);

	row = m_Scans.GetItemCount();
	m_Scans.InsertItem(row, scancode);
	if (! cases.IsEmpty()) {
		m_Scans.SetItemText(row, 1, cases);
	}
	if (! units.IsEmpty()) {
		m_Scans.SetItemText(row, 2, units);
	}
	m_Scans.EnsureVisible(row, false);
	m_Scancode.SetSel(0, -1);
	m_Scancode.SetFocus();
	return true;
}

void CRattailCEView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	CRattailCEDoc* doc;
	int i;
	CRattailScanDatum* datum;
	
	doc = GetDocument();
	m_Scans.DeleteAllItems();
	for (i = 0; i < doc->scandata.GetSize(); i++) {
		datum = (CRattailScanDatum*) doc->scandata.GetAt(i);
		m_Scans.InsertItem(i, datum->sScancode);
		m_Scans.SetItemText(i, 1, datum->sCaseQuantity);
		m_Scans.SetItemText(i, 2, datum->sUnitQuantity);
	}
	CFormView::OnUpdate(pSender, lHint, pHint);
}

BOOL CRattailCEView::OpenScanner()
{
	DWORD dwResult;
	HANDLE FindHandle;
	SCAN_FINDINFO ScanFindInfo;
	DECODER_PARAMS DecoderParams;

	SI_INIT(&ScanFindInfo);
	dwResult = SCAN_FindFirst(&ScanFindInfo, &FindHandle);
	if (dwResult != E_SCN_SUCCESS)
		return FALSE;

	dwResult = SCAN_FindClose(FindHandle);
	if (dwResult != E_SCN_SUCCESS)
		return FALSE;

	dwResult = SCAN_Open(ScanFindInfo.szDeviceName, &m_hScanner);
	if (dwResult != E_SCN_SUCCESS)
		return FALSE;

	dwResult = SCAN_Enable(m_hScanner);
	if (dwResult != E_SCN_SUCCESS) {
		SCAN_Close(m_hScanner);
		m_hScanner = NULL;
		return FALSE;
	}

	SI_INIT(&DecoderParams);
	dwResult = SCAN_GetDecoderParams(m_hScanner, DECODER_UPCE0, &DecoderParams);
	if (dwResult != E_SCN_SUCCESS) {
		SCAN_Disable(m_hScanner);
		SCAN_Close(m_hScanner);
		m_hScanner = NULL;
		return FALSE;
	}

	SI_SET_FIELD(&DecoderParams, dec_specific.upce0_params.bConvertToUPCA, TRUE);
	dwResult = SCAN_SetDecoderParams(m_hScanner, DECODER_UPCE0, &DecoderParams);
	if (dwResult != E_SCN_SUCCESS) {
		SCAN_Disable(m_hScanner);
		SCAN_Close(m_hScanner);
		m_hScanner = NULL;
		return FALSE;
	}

	// TODO:  I already don't remember where I got this 7095 figure...
	m_lpScanBuffer = SCAN_AllocateBuffer(TRUE, 7095);
	return TRUE;
}

void CRattailCEView::CloseScanner()
{
	if (m_lpScanBuffer) {
		SCAN_DeallocateBuffer(m_lpScanBuffer);
		m_lpScanBuffer = NULL;
	}
	if (m_hScanner != NULL) {
		SCAN_Close(m_hScanner);
		m_hScanner = NULL;
	}
}

BOOL CRattailCEView::EnableScanner()
{
	DWORD dwResult;

	if (! m_hScanner)
		return FALSE;

	dwResult = SCAN_ReadLabelMsg(m_hScanner, m_lpScanBuffer, m_hWnd, UM_SCAN, 0, &m_dwRequestID);
	if (dwResult != E_SCN_SUCCESS)
		return FALSE;

	return TRUE;
}

void CRattailCEView::DisableScanner()
{
	if (m_dwRequestID) {
		SCAN_CancelRead(m_hScanner, m_dwRequestID);
		m_dwRequestID = NULL;
	}
}

void CRattailCEView::OnScancodeSetfocus() 
{
	EnableScanner();
}

void CRattailCEView::OnScancodeKillfocus() 
{
	DisableScanner();
}
